@@ -1,153 +1,3 @@
-## AddVarWidget <- function(ExistingItems=NULL,container=NULL,title=NULL,ID=NULL){
-##   ## modified from RGtk2 package
-##   ## ExistingItems: existing data set for a case/file etc. It is data frame of 2 columns, the first is Variable
-##   ## container: similar to that of gWidget package.
-##   COLUMN <- c(Variable = 0, Value = 1,  editable = 2)
-##   articles <- NULL
-
-##   create.model <- function()
-##     {
-##       ## create the array of data
-##       articles <<- list()
-##       ##  create list store
-##       model <- gtkListStoreNew( "gchararray", "gchararray", "gboolean")
-##       ## add item from ExistingItems
-##       ## needs modification
-##       if (!is.null(ExistingItems)){
-##         articles <<- c(articles,unlist(apply(ExistingItems,1,function(x) list(list(Variable=x[1],Value=x[2],editable=TRUE))),FALSE))
-##         for (i in 1:length(articles))
-##           {
-##             iter <- model$append()$iter
-##             model$set(iter, COLUMN["Variable"], articles[[i]]$Variable,
-##                       COLUMN["Value"], articles[[i]]$Value,
-##                       COLUMN["editable"], articles[[i]]$editable)
-##           }
-##       }
-##       return(model)
-##     }
-  
-##   add.item <- function(button, data)
-##     {
-##       stopifnot(!is.null(articles))
-##       foo <- list(Variable = "New Var Name", Value = "NA", editable = TRUE)
-##       articles <<- c(articles, foo)
-##       iter <- model$append()$iter
-##       model$set(iter, COLUMN["Variable"], foo$Variable,
-##                 COLUMN["Value"], foo$Value,
-##                 COLUMN["editable"], foo$editable)
-##     }
-  
-##   remove.item <- function(widget, data)
-##     {
-##       checkPtrType(data, "GtkTreeView")
-##       treeview <- data
-##       model <- treeview$getModel()
-##       selection <- treeview$getSelection()
-##       selected <- selection$getSelected()
-##       if (selected[[1]])
-##         {
-##           iter <- selected$iter
-##           path <- model$getPath(iter)
-##           i <- path$getIndices()[[1]]
-##           model$remove(iter)
-##           articles <<- articles[-i]
-##         }
-##     }
-  
-##   cell.edited <- function(cell, path.string, new.text, data)
-##     {
-##       checkPtrType(data, "GtkListStore")
-##       model <- data
-##       path <- gtkTreePathNewFromString(path.string) 
-##       column <- cell$getData("column")
-##       iter <- model$getIter(path)$iter
-##       switch(column+1,
-##              {
-##                old.text <- model$get(iter, column)
-##                i <- path$getIndices()[[1]]+1
-##                articles[[i]]$Variable <<- new.text
-##                model$set(iter, column, articles[[i]]$Variable)
-##              },
-##              {
-##                i <- path$getIndices()[[1]]+1
-##                articles[[i]]$Value <<- new.text
-##                model$set(iter, column, articles[[i]]$Value)
-##              }
-##              )
-##     }
-  
-##   add.columns <- function(treeview)
-##     {
-##       model <- treeview$getModel()
-##       ## Variable column
-##       renderer <- gtkCellRendererTextNew()
-##       gSignalConnect(renderer, "edited", cell.edited, model)
-##       renderer$setData("column", COLUMN["Variable"])
-##       treeview$insertColumnWithAttributes(-1, "Variable", renderer,text = COLUMN[["Variable"]], editable = COLUMN[["editable"]])
-##       ## Value column
-##       renderer <- gtkCellRendererTextNew()
-##       gSignalConnect(renderer, "edited", cell.edited, model)
-##       renderer$setData("column", COLUMN["Value"])
-##       treeview$insertColumnWithAttributes(-1, "Value", renderer, text = COLUMN[["Value"]],editable = COLUMN[["editable"]])
-##     }
-  
-##   save.project <- function(button,data){
-##     ## push dataset into project file.
-##      IterFirst <- data$getIterFirst()
-##      cond <- IterFirst[[1]]
-##      iter <- IterFirst$iter
-##      ans <- c()
-##      while(cond) {
-##        dat <- unlist(data$get(iter, 0, 1))
-##        ans <- c(ans,dat)
-##        cond <- data$iterNext(iter)
-##      }
-##      n <- length(ans)
-##      idx1 <- seq(1,to=n,by=2)
-##      idx2 <- seq(2,to=n,by=2)
-##      ans <- data.frame(Variable=ans[idx1],Value=ans[idx2])
-##      ans <- cbind(ans,ID)
-##      dbGetQuery(.rqda$qdacon,sprintf("delete from caseAttr where caseid='%s'",ID))
-##      dbWriteTable(.rqda$qdacon, "caseAttr", ans, append = TRUE,row.names=FALSE)
-##      window$Destroy()## close
-##    }
-  
-##   ## create window, etc
-##   window <- gtkWindowNew("toplevel", show = F)
-##   window$setTitle(paste("Var:",title))
-##   window$setBorderWidth(5)
-##   vbox <- gtkVBoxNew(FALSE, 5)
-##   window$add(vbox)
-##   sw <- gtkScrolledWindowNew(NULL, NULL)
-##   sw$setShadowType("etched-in")
-##   sw$setPolicy("automatic", "automatic")
-##   vbox$packStart(sw, TRUE, TRUE, 0)
-##   ## create model
-##   model <- create.model()
-##   ## create tree view
-##   treeview <- gtkTreeViewNewWithModel(model)
-##   treeview$setRulesHint(TRUE)
-##   treeview$getSelection()$setMode("single")
-##   add.columns(treeview)
-##   sw$add(treeview)
-##   ## some buttons
-##   hbox <- gtkHBoxNew(TRUE, 4)
-##   vbox$packStart(hbox, FALSE, FALSE, 0)
-##   button <- gtkButtonNewWithLabel("Add")
-##   gSignalConnect(button, "clicked", add.item, model)
-##   hbox$packStart(button, TRUE, TRUE, 0)
-##   button <- gtkButtonNewWithLabel("Remove")
-##   gSignalConnect(button, "clicked", remove.item, treeview)
-##   hbox$packStart(button, TRUE, TRUE, 0)
-##   button <- gtkButtonNewWithLabel("Save")
-##   gSignalConnect(button, "clicked",save.project,model)
-##   hbox$packStart(button, TRUE, TRUE, 0)
-##   window$setDefaultSize(150, 350)
-##   window$showAll()
-##   invisible(window)
-## }
-
-
 EditVarWidget <- function(ExistingItems=NULL,container=NULL,title=NULL,ID=NULL,saveFUN=NULL,...){
   ## modified from RGtk2 package
   ## ExistingItems: existing data set for a case/file etc. It is data frame of 2 columns, the first is Variable
@@ -181,7 +31,7 @@ EditVarWidget <- function(ExistingItems=NULL,container=NULL,title=NULL,ID=NULL,s
     {
       checkPtrType(data, "GtkListStore")
       model <- data
-      path <- gtkTreePathNewFromString(path.string) 
+      path <- gtkTreePathNewFromString(path.string)
       column <- cell$getData("column")
       iter <- model$getIter(path)$iter
       if (column==1){
@@ -190,7 +40,7 @@ EditVarWidget <- function(ExistingItems=NULL,container=NULL,title=NULL,ID=NULL,s
                model$set(iter, column, articles[[i]]$Value)
              }
     }
-  
+
   add.columns <- function(treeview)
     {
       model <- treeview$getModel()
@@ -207,7 +57,7 @@ EditVarWidget <- function(ExistingItems=NULL,container=NULL,title=NULL,ID=NULL,s
     }
 
     saveFUN <- get(saveFUN,mode="function")
- 
+
   ## create window, etc
   window <- gtkWindowNew("toplevel", show = F)
   window$setTitle(paste("Var:",title))
@@ -261,11 +111,11 @@ saveFUN4CaseAttr <- function(button,data){
     ## cal which variable is added and which is modified
     change_idx <- ans$Value != ExistingItems$value
     mod_idx <- change_idx & (ExistingItems$value!= "NA")
-    new_idx <- change_idx & (! mod_idx)   
+    new_idx <- change_idx & (! mod_idx)
     if (any(mod_idx)) {
     ## alter the table for the modified variable
     vars <- ans[mod_idx,]
-    apply(vars,1,FUN=function(x) dbGetQuery(.rqda$qdacon,sprintf("update caseAttr set value == '%s' where variable == '%s' and caseID=='%s' status==1",x[2],x[1],MoreArgs$caseId)))
+    apply(vars,1,FUN=function(x) dbGetQuery(.rqda$qdacon,sprintf("update caseAttr set value == '%s' where variable == '%s' and caseID=='%s' and status==1",x[2],x[1],MoreArgs$caseId)))
     }
     if (any(new_idx)){
     ## add the new variable to table
@@ -318,11 +168,11 @@ saveFUN4FileAttr <- function(button,data){
     if (any(mod_idx)) {
     ## alter the table for the modified variable
     vars <- ans[mod_idx,]
-    apply(vars,1,FUN=function(x) dbGetQuery(.rqda$qdacon,sprintf("update fileAttr set value == '%s' where variable == '%s' and fileID=='%s' status==1",x[2],x[1],MoreArgs$fileId)))
+    apply(vars,1,FUN=function(x) dbGetQuery(.rqda$qdacon,sprintf("update fileAttr set value == '%s' where variable == '%s' and fileID=='%s'and status==1",x[2],x[1],MoreArgs$fileId)))
     }
     if (any(new_idx)){
     ## add the new variable to table
-    vars <- data.frame(variable=ans[new_idx,1],value=ans[new_idx,2],fileID=MoreArgs$fileId,date=date(),dateM=NA,owner=.rqda$owner,status==1)
+    vars <- data.frame(variable=ans[new_idx,1],value=ans[new_idx,2],fileID=MoreArgs$fileId,date=date(),dateM=NA,owner=.rqda$owner,status=1)
     dbWriteTable(.rqda$qdacon, "fileAttr", vars, append = TRUE,row.names=FALSE)
     }
   }
@@ -375,79 +225,86 @@ AddAttrNames <- function(name,...) {
 }
 
 AddAttrButton <- function(label="ADD"){
-  gbutton(label,handler=function(h,...) {
-    if (is_projOpen(env=.rqda,conName="qdacon")) {
-      AttrName <- ginput("Enter new Attr Name. ", icon="info")
-      if (!is.na(AttrName)) {
-        AttrName <- enc(AttrName,encoding="UTF-8")
-        ## Encoding(AttrName) <- "UTF-8"
-        if (AttrName %in% c("fileID","caseID")) gmessage("This is a reserved keyword.",con=TRUE) else{
-        AddAttrNames(AttrName)
-        AttrNamesUpdate()
-      }}
-    }
-  }
-          )
-}
-
-DeleteAttrButton <- function(label="Delete"){
-  gbutton(label,handler=function(h,...) {
-    if (is_projOpen(env=.rqda,conName="qdacon") & length(svalue(.rqda$.AttrNamesWidget))!=0)
-          {
-            del <- gconfirm("Really delete the Attribute?",icon="question")
-            if (isTRUE(del)){
-              Selected <- svalue(.rqda$.AttrNamesWidget)
-              ## Encoding(Selected) <- "UTF-8"
-              Selected <- enc(Selected,"UTF-8")
-              dbGetQuery(.rqda$qdacon,sprintf("update attributes set status=0 where name=='%s'",Selected))
-              ##dbGetQuery(.rqda$qdacon,sprintf("delete from caseAttr where variable=='%s'",Selected))
-              RQDAQuery(sprintf("update caseAttr set status==0 where variable=='%s'",Selected))
-              ##dbGetQuery(.rqda$qdacon,sprintf("delete from fileAttr where variable=='%s'",Selected))
-              RQDAQuery(sprintf("update fileAttr set status==0 where variable=='%s'",Selected))
-              AttrNamesUpdate()
-            }
-          }
-  }
-          )
-}
-
-RenameAttrButton <- function(label="Rename"){
-  gbutton(label,handler=function(h,...) {
-    if (is_projOpen(env=.rqda,conName="qdacon")) {
-      selected <- svalue(.rqda$.AttrNamesWidget)
-      if (length(selected)==0){
-        gmessage("Select a attribute first.",icon="error",con=TRUE)
-      }
-      else {
-        ## get the new file names
-        NewName <- ginput("Enter new attribute name. ", text=selected, icon="info")
-        if (!is.na(NewName)){
-          ## Encoding(NewName) <- "UTF-8"
-          selected <- enc(selected,encoding="UTF-8")
-          NewName<- enc(NewName,encoding="UTF-8")
-          exists <- dbGetQuery(.rqda$qdacon, sprintf("select * from attributes where name == '%s' ",NewName))
-          if (nrow(exists) > 0 ){
-          gmessage("Name duplicated. Please use anaother name.",cont=TRUE)
-          } else {
-          dbGetQuery(.rqda$qdacon, sprintf("update attributes set name = '%s' where name == '%s' ",NewName,selected))
-          dbGetQuery(.rqda$qdacon, sprintf("update caseAttr set variable = '%s' where variable == '%s' ",NewName,selected))
-          dbGetQuery(.rqda$qdacon, sprintf("update fileAttr set variable = '%s' where variable == '%s' ",NewName,selected))
+  AddAttB <- gbutton(label,handler=function(h,...) {
+    AttrName <- ginput("Enter new Attr Name. ", icon="info")
+    if (!is.na(AttrName)) {
+      Encoding(AttrName) <- "UTF-8"
+      invalid <- grepl("'",AttrName)
+      if (invalid) {
+        gmessage("Attribute should NOT contain '.",con=TRUE)
+      } else {
+        if (AttrName %in% c("fileID","caseID")) {
+          gmessage("This is a reserved keyword.",con=TRUE)
+        } else{
+          AddAttrNames(AttrName)
           AttrNamesUpdate()
-         }
         }
       }
     }
   }
-          )
+                     )
+  assign("AddAttB",AddAttB,env=button)
+  enabled(AddAttB) <- FALSE
+  AddAttB
+}
+
+
+DeleteAttrButton <- function(label="Delete"){
+  DelAttB <- gbutton(label,handler=function(h,...) {
+    del <- gconfirm("Really delete the Attribute?",icon="question")
+    if (isTRUE(del)){
+      Selected <- svalue(.rqda$.AttrNamesWidget)
+      Selected <- enc(Selected,"UTF-8")
+      dbGetQuery(.rqda$qdacon,sprintf("update attributes set status=0 where name=='%s'",Selected))
+      RQDAQuery(sprintf("update caseAttr set status==0 where variable=='%s'",Selected))
+      RQDAQuery(sprintf("update fileAttr set status==0 where variable=='%s'",Selected))
+      AttrNamesUpdate()
+    }
+  }
+                     )
+  assign("DelAttB",DelAttB,env=button)
+  enabled(DelAttB) <- FALSE
+  DelAttB
+}
+
+
+RenameAttrButton <- function(label="Rename"){
+  RenAttB <- gbutton(label,handler=function(h,...) {
+    selected <- svalue(.rqda$.AttrNamesWidget)
+    NewName <- ginput("Enter new attribute name. ", text=selected, icon="info")
+    if (!is.na(NewName)){
+      Encoding(NewName) <- "UTF-8"
+      selected <- enc(selected,encoding="UTF-8")
+      invalid <- grepl("'",NewName)
+      if (invalid) {
+        gmessage("Attribute should NOT contain '.",con=TRUE)
+      } else {
+        exists <- dbGetQuery(.rqda$qdacon, sprintf("select * from attributes where name == '%s' ",NewName))
+        if (nrow(exists) > 0 ){
+          gmessage("Name duplicated. Please use anaother name.",cont=TRUE)
+        } else {
+          dbGetQuery(.rqda$qdacon, sprintf("update attributes set name = '%s' where name == '%s' ",NewName,selected))
+          dbGetQuery(.rqda$qdacon, sprintf("update caseAttr set variable = '%s' where variable == '%s' ",NewName,selected))
+          dbGetQuery(.rqda$qdacon, sprintf("update fileAttr set variable = '%s' where variable == '%s' ",NewName,selected))
+          AttrNamesUpdate()
+        }
+      }
+    }
+  }
+                     )
+  assign("RenAttB",RenAttB,env=button)
+  enabled(RenAttB) <- FALSE
+  RenAttB
 }
 
 AttrMemoButton <- function(label="Memo"){
-  gbutton(label,handler=function(h,...) {
-    if (is_projOpen(env=.rqda,conName="qdacon")) {
+  AttMemB <- gbutton(label,handler=function(h,...) {
     MemoWidget("Attributes",.rqda$.AttrNamesWidget,"attributes")
- }
   }
-          )
+                     )
+  assign("AttMemB",AttMemB,env=button)
+  enabled(AttMemB) <- FALSE
+  AttMemB
 }
 
 viewCaseAttr <- function(){
@@ -477,7 +334,7 @@ viewFileAttr <- function(){
 }
 
 
-GetAttr <- function(type=c("case","file"),attrs=svalue(.rqda$.AttrNamesWidget)){
+GetAttr <- function(type=c("case","file"),attrs=svalue(.rqda$.AttrNamesWidget),subset){
   if (isIdCurrent(.rqda$qdacon)){
   type <-  match.arg(type)
   if (length(attrs)==0) attrs <- NULL
@@ -502,7 +359,7 @@ GetAttr <- function(type=c("case","file"),attrs=svalue(.rqda$.AttrNamesWidget)){
     fileName <- dbGetQuery(.rqda$qdacon,"select name,id from source where status==1")
     if (nrow(fileName)!=0){
       names(fileName) <- c("file","fileID")
-      Encoding(fileName$case) <- "UTF-8"
+      Encoding(fileName$file) <- "UTF-8"
       DF <- merge(fileName,DF)
       class(DF) <- c("FileAttr","data.frame")
     }}
@@ -511,17 +368,25 @@ GetAttr <- function(type=c("case","file"),attrs=svalue(.rqda$.AttrNamesWidget)){
   attrs <- tt[tt$class=="numeric","name"]
   idx <- which(names(DF) %in% attrs)
   DF[,idx]<-as.data.frame(apply(DF[,idx,drop=FALSE],2,as.numeric))
-  DF
+  if (missing(subset)) DF else {
+      r <- eval(substitute(subset),DF)
+      if (!is.logical(r))
+          stop("'subset' must evaluate to logical")
+      r <- r & !is.na(r)
+      DF <- DF[r,,drop=FALSE]
+      DF
+  }
 }}
 
 SetAttrClsButton <- function(label="Class"){
-    ans <- gbutton(label,handler=function(h,...) {
-        if (is_projOpen(env=.rqda,conName="qdacon")) {
-            setAttrType()
-        }
-    })
+  ans <- gbutton(label,handler=function(h,...) {
+    setAttrType()
+  }
+                 )
   gtkTooltips()$setTip(ans@widget@widget,"Set class of selected attribute.\nIt can be 'numeric' or 'character'.")
-  return(ans)
+  assign("SetAttClsB",ans,env=button)
+  enabled(ans) <- FALSE
+  ans
 }
 
 
@@ -548,3 +413,158 @@ setAttrType <- function() {
       }
       dispose(w)
     })}
+
+
+
+#######################
+## Defunct functions
+#######################
+
+## AddVarWidget <- function(ExistingItems=NULL,container=NULL,title=NULL,ID=NULL){
+##   ## modified from RGtk2 package
+##   ## ExistingItems: existing data set for a case/file etc. It is data frame of 2 columns, the first is Variable
+##   ## container: similar to that of gWidget package.
+##   COLUMN <- c(Variable = 0, Value = 1,  editable = 2)
+##   articles <- NULL
+
+##   create.model <- function()
+##     {
+##       ## create the array of data
+##       articles <<- list()
+##       ##  create list store
+##       model <- gtkListStoreNew( "gchararray", "gchararray", "gboolean")
+##       ## add item from ExistingItems
+##       ## needs modification
+##       if (!is.null(ExistingItems)){
+##         articles <<- c(articles,unlist(apply(ExistingItems,1,function(x) list(list(Variable=x[1],Value=x[2],editable=TRUE))),FALSE))
+##         for (i in 1:length(articles))
+##           {
+##             iter <- model$append()$iter
+##             model$set(iter, COLUMN["Variable"], articles[[i]]$Variable,
+##                       COLUMN["Value"], articles[[i]]$Value,
+##                       COLUMN["editable"], articles[[i]]$editable)
+##           }
+##       }
+##       return(model)
+##     }
+
+##   add.item <- function(button, data)
+##     {
+##       stopifnot(!is.null(articles))
+##       foo <- list(Variable = "New Var Name", Value = "NA", editable = TRUE)
+##       articles <<- c(articles, foo)
+##       iter <- model$append()$iter
+##       model$set(iter, COLUMN["Variable"], foo$Variable,
+##                 COLUMN["Value"], foo$Value,
+##                 COLUMN["editable"], foo$editable)
+##     }
+
+##   remove.item <- function(widget, data)
+##     {
+##       checkPtrType(data, "GtkTreeView")
+##       treeview <- data
+##       model <- treeview$getModel()
+##       selection <- treeview$getSelection()
+##       selected <- selection$getSelected()
+##       if (selected[[1]])
+##         {
+##           iter <- selected$iter
+##           path <- model$getPath(iter)
+##           i <- path$getIndices()[[1]]
+##           model$remove(iter)
+##           articles <<- articles[-i]
+##         }
+##     }
+
+##   cell.edited <- function(cell, path.string, new.text, data)
+##     {
+##       checkPtrType(data, "GtkListStore")
+##       model <- data
+##       path <- gtkTreePathNewFromString(path.string)
+##       column <- cell$getData("column")
+##       iter <- model$getIter(path)$iter
+##       switch(column+1,
+##              {
+##                old.text <- model$get(iter, column)
+##                i <- path$getIndices()[[1]]+1
+##                articles[[i]]$Variable <<- new.text
+##                model$set(iter, column, articles[[i]]$Variable)
+##              },
+##              {
+##                i <- path$getIndices()[[1]]+1
+##                articles[[i]]$Value <<- new.text
+##                model$set(iter, column, articles[[i]]$Value)
+##              }
+##              )
+##     }
+
+##   add.columns <- function(treeview)
+##     {
+##       model <- treeview$getModel()
+##       ## Variable column
+##       renderer <- gtkCellRendererTextNew()
+##       gSignalConnect(renderer, "edited", cell.edited, model)
+##       renderer$setData("column", COLUMN["Variable"])
+##       treeview$insertColumnWithAttributes(-1, "Variable", renderer,text = COLUMN[["Variable"]], editable = COLUMN[["editable"]])
+##       ## Value column
+##       renderer <- gtkCellRendererTextNew()
+##       gSignalConnect(renderer, "edited", cell.edited, model)
+##       renderer$setData("column", COLUMN["Value"])
+##       treeview$insertColumnWithAttributes(-1, "Value", renderer, text = COLUMN[["Value"]],editable = COLUMN[["editable"]])
+##     }
+
+##   save.project <- function(button,data){
+##     ## push dataset into project file.
+##      IterFirst <- data$getIterFirst()
+##      cond <- IterFirst[[1]]
+##      iter <- IterFirst$iter
+##      ans <- c()
+##      while(cond) {
+##        dat <- unlist(data$get(iter, 0, 1))
+##        ans <- c(ans,dat)
+##        cond <- data$iterNext(iter)
+##      }
+##      n <- length(ans)
+##      idx1 <- seq(1,to=n,by=2)
+##      idx2 <- seq(2,to=n,by=2)
+##      ans <- data.frame(Variable=ans[idx1],Value=ans[idx2])
+##      ans <- cbind(ans,ID)
+##      dbGetQuery(.rqda$qdacon,sprintf("delete from caseAttr where caseid='%s'",ID))
+##      dbWriteTable(.rqda$qdacon, "caseAttr", ans, append = TRUE,row.names=FALSE)
+##      window$Destroy()## close
+##    }
+
+##   ## create window, etc
+##   window <- gtkWindowNew("toplevel", show = F)
+##   window$setTitle(paste("Var:",title))
+##   window$setBorderWidth(5)
+##   vbox <- gtkVBoxNew(FALSE, 5)
+##   window$add(vbox)
+##   sw <- gtkScrolledWindowNew(NULL, NULL)
+##   sw$setShadowType("etched-in")
+##   sw$setPolicy("automatic", "automatic")
+##   vbox$packStart(sw, TRUE, TRUE, 0)
+##   ## create model
+##   model <- create.model()
+##   ## create tree view
+##   treeview <- gtkTreeViewNewWithModel(model)
+##   treeview$setRulesHint(TRUE)
+##   treeview$getSelection()$setMode("single")
+##   add.columns(treeview)
+##   sw$add(treeview)
+##   ## some buttons
+##   hbox <- gtkHBoxNew(TRUE, 4)
+##   vbox$packStart(hbox, FALSE, FALSE, 0)
+##   button <- gtkButtonNewWithLabel("Add")
+##   gSignalConnect(button, "clicked", add.item, model)
+##   hbox$packStart(button, TRUE, TRUE, 0)
+##   button <- gtkButtonNewWithLabel("Remove")
+##   gSignalConnect(button, "clicked", remove.item, treeview)
+##   hbox$packStart(button, TRUE, TRUE, 0)
+##   button <- gtkButtonNewWithLabel("Save")
+##   gSignalConnect(button, "clicked",save.project,model)
+##   hbox$packStart(button, TRUE, TRUE, 0)
+##   window$setDefaultSize(150, 350)
+##   window$showAll()
+##   invisible(window)
+## }
