@@ -1,18 +1,19 @@
-CaseNamesUpdate <- function(CaseNamesWidget=.rqda$.CasesNamesWidget,sortByTime=FALSE,decreasing=FALSE,...)
+CaseNamesUpdate <- function(CaseNamesWidget=.rqda$.CasesNamesWidget,
+                            sortByTime=FALSE,decreasing=FALSE,...)
 {
   if (isIdCurrent(.rqda$qdacon)){
-##  CaseName <- dbGetQuery(.rqda$qdacon, "select name, id,date from cases where status=1 order by lower(name)")
-  CaseName <- dbGetQuery(.rqda$qdacon, "select name, id,date from cases where status=1")
-  if (nrow(CaseName)==0) {
-    case <- NULL
-  } else {
-    case <- CaseName$name
-    Encoding(case) <- "UTF-8"
-    if (!sortByTime) {case <- sort(case,decreasing=decreasing)} else {
-    case <- case[OrderByTime(CaseName$date,decreasing=decreasing)]
+    ## CaseName <- dbGetQuery(.rqda$qdacon, "select name, id,date from cases where status=1 order by lower(name)")
+    CaseName <- dbGetQuery(.rqda$qdacon, "select name, id,date from cases where status=1")
+    if (nrow(CaseName)==0) {
+      case <- NULL
+    } else {
+      case <- CaseName$name
+      Encoding(case) <- "UTF-8"
+      if (!sortByTime) {case <- sort(case,decreasing=decreasing)} else {
+        case <- case[OrderByTime(CaseName$date,decreasing=decreasing)]
+      }
     }
-  }
-     tryCatch(CaseNamesWidget[] <- case, error=function(e){})
+    tryCatch(CaseNamesWidget[] <- case, error=function(e){})
   }
 }
 
@@ -27,13 +28,13 @@ AddCase <- function(name,conName="qdacon",assignenv=.rqda,...) {
     if (nextid==1){
       write <- TRUE
     } else {
-      dup <- dbGetQuery(con,sprintf("select name from cases where name=='%s'",name))
+      dup <- dbGetQuery(con,sprintf("select name from cases where name=='%s'",enc(name)))
       if (nrow(dup)==0) write <- TRUE
     }
     if (write ) {
       dbGetQuery(con,sprintf("insert into cases (name, id, status,date,owner)
                                             values ('%s', %i, %i,%s, %s)",
-                             name,nextid, 1, shQuote(date()),shQuote(.rqda$owner)))
+                             enc(name),nextid, 1, shQuote(date()),shQuote(.rqda$owner)))
     }
   }
 }
@@ -42,9 +43,9 @@ AddCase <- function(name,conName="qdacon",assignenv=.rqda,...) {
 AddFileToCaselinkage <- function(Widget=.rqda$.fnames_rqda){
   ## filenames -> fid -> selfirst=0; selend=nchar(filesource)
   filename <- svalue(Widget)
-  Encoding(filename) <- "unknown"
+  ## Encoding(filename) <- "unknown"
   query <- dbGetQuery(.rqda$qdacon,sprintf("select id, file from source where name in (%s) and status=1",
-  paste("'",filename,"'",sep="",collapse=",")))
+  paste("'",enc(filename),"'",sep="",collapse=",")))
   fid <- query$id
   Encoding(query$file) <- "UTF-8"
   selend <- nchar(query$file)
@@ -53,7 +54,7 @@ AddFileToCaselinkage <- function(Widget=.rqda$.fnames_rqda){
   cases <- dbGetQuery(.rqda$qdacon,"select id, name from cases where status=1")
   if (nrow(cases)!=0){
     Encoding(cases$name) <- "UTF-8"
-##    ans <- select.list(cases$name,multiple=FALSE)
+    ## ans <- select.list(cases$name,multiple=FALSE)
     CurrentFrame <- sys.frame(sys.nframe())
 
     RunOnSelected(cases$name,multiple=FALSE,enclos=CurrentFrame,expr={
@@ -80,21 +81,21 @@ AddFileToCaselinkage <- function(Widget=.rqda$.fnames_rqda){
 UpdateFileofCaseWidget <- function(con=.rqda$qdacon,Widget=.rqda$.FileofCase,sortByTime=FALSE,...){
   Selected <- svalue(.rqda$.CasesNamesWidget)
   if (length(Selected)!=0){
-    caseid <- dbGetQuery(.rqda$qdacon,sprintf("select id from cases where status=1 and name='%s'",Selected))[,1]
-    ## Encoding(Selected) <- "UTF-8"
+    caseid <- dbGetQuery(.rqda$qdacon,sprintf("select id from cases where status=1 and name='%s'",
+                                              enc(Selected)))[,1]
     Total_fid <- dbGetQuery(con,sprintf("select fid from caselinkage where status==1 and caseid==%i",caseid))
     if (nrow(Total_fid)!=0){
       items <- dbGetQuery(con,"select name,id,date from source where status==1")
       if (nrow(items)!=0) {
-          if (sortByTime){
-              items <- items[items$id %in% Total_fid$fid,c("name","date")]
-              items <- items$name[OrderByTime(items$date)]
-              Encoding(items) <- "UTF-8"}
-          else{
-              items <- items[items$id %in% Total_fid$fid,c("name")]
-              Encoding(items) <- "UTF-8"
-              items <- sort(items)
-          }
+        if (sortByTime){
+          items <- items[items$id %in% Total_fid$fid,c("name","date")]
+          items <- items$name[OrderByTime(items$date)]
+          Encoding(items) <- "UTF-8"}
+        else{
+          items <- items[items$id %in% Total_fid$fid,c("name")]
+          Encoding(items) <- "UTF-8"
+          items <- sort(items)
+        }
       } else items <- NULL
     } else items <- NULL
   } else items <- NULL
@@ -104,10 +105,11 @@ UpdateFileofCaseWidget <- function(con=.rqda$qdacon,Widget=.rqda$.FileofCase,sor
 HL_Case <- function(){
   if (is_projOpen(env=.rqda,conName="qdacon")) {
     SelectedFile <- svalue(.rqda$.root_edit)
-    currentFid <-  RQDAQuery(sprintf("select id from source where name=='%s'",SelectedFile))[,1]
+    currentFid <-  RQDAQuery(sprintf("select id from source where name=='%s'",
+                                     enc(SelectedFile)))[,1]
     if (length(currentFid)!=0) {
       caseName <- svalue(.rqda$.CasesNamesWidget)
-      caseid <- RQDAQuery(sprintf("select id from cases where name=='%s'",caseName))[,1]
+      caseid <- RQDAQuery(sprintf("select id from cases where name=='%s'",enc(caseName)))[,1]
       idx <- RQDAQuery(sprintf("select selfirst,selend from caselinkage where fid=%i and status==1 and caseid=%i",currentFid,caseid))
       coding.idx <- RQDAQuery(sprintf("select selfirst,selend from coding where fid=%i and status==1",currentFid))
       anno.idx <- RQDAQuery(sprintf("select position from annotation where fid=%i and status==1",currentFid))$position
